@@ -1,7 +1,7 @@
 #ifndef GLITE_LB_LB_AUTHZ_H
 #define GLITE_LB_LB_AUTHZ_H
 
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/interface/lb_authz.h,v 1.15 2012/05/16 08:01:34 mvocu Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/interface/lb_authz.h,v 1.17 2012/09/03 10:21:32 zsustr Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -111,6 +111,11 @@ check_jobstat_authz(edg_wll_Context ctx,
                     struct _edg_wll_GssPrincipal_data *peer,
                     int *authz_flags);
 
+int
+parse_gridmap(edg_wll_Context ctx,
+              const char *file,
+              struct _edg_wll_id_mapping *mapping);
+
 #ifdef __cplusplus
 }
 #endif
