@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/bkserverd.c,v 1.144 2012/08/14 14:29:58 zsustr Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/bkserverd.c,v 1.153 2012/12/13 13:21:53 valtri Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -122,7 +122,7 @@ extern void _start (void), etext (void);
 
 /* file to store pid and generate semaphores key */
 #ifndef EDG_BKSERVERD_PIDFILE
-#define EDG_BKSERVERD_PIDFILE	"/var/run/glite-lb-bkserverd.pid"
+#define EDG_BKSERVERD_PIDFILE	"/var/run/glite/glite-lb-bkserverd.pid"
 #endif
 
 #ifndef GLITE_LBPROXY_SOCK_PREFIX
@@ -187,6 +187,8 @@ static char		**msg_brokers = NULL;
 static char		**msg_prefixes = NULL;
 char *		html_header = NULL;
 static int	html_header_forced = 0;
+static char     *gridmap = NULL;
+struct _edg_wll_id_mapping id_mapping = {NULL, 0};
 
 
 static struct option opts[] = {
@@ -239,10 +241,11 @@ static struct option opts[] = {
 	{"rss-time", 	1,	NULL,	'I'},
 	{"policy",	1,	NULL,	'l'},
 	{"exclusive-zombies-off",	0,	NULL,	'E'},
+	{"gridmap-file",1,	NULL,   'M'},
 	{NULL,0,NULL,0}
 };
 
-static const char *get_opt_string = "Ac:k:C:V:p:a:drm:ns:i:S:D:J:jR:F:xOL:N:X:Y:T:t:e:f:zb:gPBo:q:W:Z:GI:l:EH:"
+static const char *get_opt_string = "Ac:k:C:V:p:a:drm:M:ns:i:S:D:J:jR:F:xOL:N:X:Y:T:t:e:f:zb:gPBo:q:W:Z:GI:l:EH:"
 #ifdef GLITE_LB_SERVER_WITH_WS
 	"w:"
 #endif
@@ -272,6 +275,7 @@ static void usage(char *me)
 		"\t-s, --slaves\t number of slave servers to fork\n"
 		"\t-i, --pidfile\t file to store master pid\n"
 		"\t-L, --limits\t query limits numbers in format \"events_limit:jobs_limit:size_limit\"\n"
+		"\t-M, --gridmap-file\tgridmap-file to map clients identities\"\n"
 		"\t-N, --notif-dur default[:max]\t Duration of notification registrations in seconds (default and maximal)\n"
 		"\t-S, --purge-prefix\t purge files full-path prefix\n"
 		"\t-D, --dump-prefix\t dump files full-path prefix\n"
@@ -400,7 +404,7 @@ int main(int argc, char *argv[])
 {
 	int			i;
 	char *portstr = NULL;
-	int					opt, pidfile_forced = 0;
+	int					opt;
 	char				pidfile[PATH_MAX] = EDG_BKSERVERD_PIDFILE,
 					   *name;
 #ifdef GLITE_LB_SERVER_WITH_WS
@@ -468,6 +472,7 @@ int main(int argc, char *argv[])
 				return 1;
 			}
 			break;
+		case 'M': gridmap = strdup(optarg); break;
 		case 'N': {
 				int	std,max;
 				switch (sscanf(optarg,"%d:%d",&std,&max)) {
@@ -482,7 +487,7 @@ int main(int argc, char *argv[])
 			}  break;
 		case 'X': notif_ilog_socket_path = strdup(optarg); break;
 		case 'Y': notif_ilog_file_prefix = strdup(optarg); break;
-		case 'i': strcpy(pidfile,optarg); pidfile_forced = 1; break;
+		case 'i': strcpy(pidfile,optarg); break;
 		case 'R': add_root(ctx, optarg, ADMIN_ACCESS); break;
 		case 'F': msg_conf = strdup(optarg); break;
 		case 'H': html_header = strdup(optarg); html_header_forced = 1; break;
@@ -550,9 +555,6 @@ int main(int argc, char *argv[])
 		slaves = 2;
 	}
 
-	if (!pidfile_forced && geteuid())
-		snprintf(pidfile,sizeof pidfile, "%s/glite-lb-bkserverd.pid", getenv("HOME"));
-
 	fpid = fopen(pidfile,"r");
 	if ( fpid )
 	{
@@ -585,10 +587,18 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (gridmap && parse_gridmap(ctx, gridmap, &id_mapping)) {
+		char *et, *ed;
+
+		edg_wll_Error(ctx,&et,&ed);
+		glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_FATAL, "Cannot load identity mapping: %s: %s\n", et, ed);
+		return 1;
+	}
+
 	if (!html_header) {
 		char *html_header_prefix = getenv("GLITE_LB_LOCATION_ETC");
-		if (!html_header_prefix) html_header_prefix="/etc/glite-lb";
-		asprintf(&html_header, "%s/html-header.html", html_header_prefix);
+		if (!html_header_prefix) html_header_prefix="/etc";
+		asprintf(&html_header, "%s/glite-lb/html-header.html", html_header_prefix);
 	}
 	if (html_header) {
 		FILE *fp = fopen(html_header, "r");
@@ -597,6 +607,7 @@ int main(int argc, char *argv[])
 			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_DEBUG, "Using HTML header file %s", html_header);
 		} else {
 			glite_common_log(LOG_CATEGORY_CONTROL, html_header_forced ? LOG_PRIORITY_ERROR : LOG_PRIORITY_INFO, "Cannot open HTML header file %s", html_header);
+			free(html_header), html_header = NULL;
 		}
 	}
 
@@ -700,7 +711,7 @@ int main(int argc, char *argv[])
 		ret = edg_wll_gss_watch_creds(server_cert, &cert_mtime);
 		if (ret < 0)
         		glite_common_log(LOG_CATEGORY_SECURITY,LOG_PRIORITY_WARN,"edg_wll_gss_watch_creds failed, unable to access credentials\n");
-		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &mycred, &gss_code) && mycred->name != NULL)
+		if ( !edg_wll_gss_acquire_cred(server_cert, server_key, GSS_C_ACCEPT, &mycred, &gss_code) && mycred->name != NULL)
 		{
 			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_INFO, "Server identity: %s", mycred->name);
 			server_subject = strdup(mycred->name);
@@ -1020,14 +1031,14 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 				*name_num = NULL,
 				*name = NULL;
 	int					h_errno, ret;
-	int			npref, totpref;
+	int			npref, totpref, i;
 
 
 #if 0
 	switch ( edg_wll_gss_watch_creds(server_cert, &cert_mtime) ) {
 	case 0: break;
 	case 1:
-		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &newcred, &gss_code) ) {
+		if ( !edg_wll_gss_acquire_cred(server_cert, server_key, GSS_C_ACCEPT, &newcred, &gss_code) ) {
 			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_INFO, "[%d] reloading credentials successful", getpid());
 			edg_wll_gss_release_cred(&mycred, NULL);
 			mycred = newcred;
@@ -1040,7 +1051,7 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 		break;
 	}
 #else
-		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &newcred, &gss_code) ) {
+	if ( !edg_wll_gss_acquire_cred(server_cert, server_key, GSS_C_ACCEPT, &newcred, &gss_code) ) {
 			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_INFO, "[%d] reloading credentials successful", getpid());
 			edg_wll_gss_release_cred(&mycred, NULL);
 			mycred = newcred;
@@ -1105,6 +1116,14 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 
 	if (policy_file) ctx->authz_policy_file = strdup(policy_file);
 	if (html_header) ctx->html_header_file = strdup(html_header);
+	else ctx->html_header_file = NULL;
+
+	ctx->id_mapping.num = id_mapping.num;
+	ctx->id_mapping.rules = (_edg_wll_mapping_rule*)malloc(ctx->id_mapping.num * sizeof(_edg_wll_mapping_rule));
+	for ( i = 0; i < ctx->id_mapping.num; i++ ) {
+		ctx->id_mapping.rules[i].a = strdup(id_mapping.rules[i].a);
+		ctx->id_mapping.rules[i].b = strdup(id_mapping.rules[i].b);
+	}
 
 	gettimeofday(&conn_start, 0);
 
@@ -1833,6 +1852,7 @@ void callback_ares_getnameinfo(void *arg, int status, char *node, char *service)
 			arp->err = NO_DATA;
 		}
 		break;
+	    case ARES_ENODATA:
 	    case ARES_EBADNAME:
 	    case ARES_ENOTFOUND:
 		arp->err = HOST_NOT_FOUND;
