@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/jobstat.c,v 1.103 2012/08/01 13:41:06 zsustr Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/jobstat.c,v 1.108 2012/09/06 10:24:36 jfilipov Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -15,7 +15,7 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/jobstat.c,v 1.103 2012/08/01 13:41:06 zsustr Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/jobstat.c,v 1.108 2012/09/06 10:24:36 jfilipov Exp $"
 
 #include <inttypes.h>
 #include <stdio.h>
@@ -28,6 +28,7 @@ limitations under the License.
 
 #include "glite/lbu/trio.h"
 #include "glite/lb/events.h"
+#include "glite/lb/events_json.h"
 #include "glite/lb/context-int.h"
 #include "glite/lb/intjobstat.h"
 #include "glite/lb/process_event.h"
@@ -59,7 +60,6 @@ limitations under the License.
 
 #define mov(a,b) { free(a); a = b; b = NULL; }
 
-static void warn (const char* format, ...) UNUSED_VAR ;
 static char *job_owner(edg_wll_Context,char *);
 static edg_wll_ErrorCode get_job_parent(edg_wll_Context ctx, glite_jobid_const_t job, glite_jobid_t *parent);
 
@@ -90,6 +90,106 @@ static char* matched_substr(char *in, regmatch_t match)
 	return s;
 }
 
+
+edg_wll_Event* fetch_history(edg_wll_Context ctx, edg_wll_JobStat *stat) {
+        edg_wll_QueryRec         jc0[2], *jc[2];
+        edg_wll_Event *events = NULL;
+
+        jc[0] = jc0;
+        jc[1] = NULL;
+        jc[0][0].attr = EDG_WLL_QUERY_ATTR_JOBID;
+        jc[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
+        jc[0][0].value.j = stat->jobId;
+        jc[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
+
+        edg_wll_QueryEventsServer(ctx, 1, (const edg_wll_QueryRec **)jc, NULL, &events);
+
+        return events;
+}
+
+
+int collate_history(edg_wll_Context ctx, edg_wll_JobStat *stat, edg_wll_Event* events, int authz_flags) {
+        char                    *event_str = NULL, *history = NULL;
+        void *tmpptr;
+        size_t i;
+        size_t  size, len, maxsize = 1024, newsize;
+        char *olduser;
+        char *oldacluser;
+
+        if (!events || !events[0].type) {
+                history = strdup(HISTORY_EMPTY);
+        } else {
+                history = malloc(maxsize);
+                strcpy(history, HISTORY_HEADER);
+                size = HISTORY_HEADER_SIZE;
+
+                oldacluser = NULL;
+                olduser = NULL;
+                for (i = 0; events && events[i].type; i++) {
+
+                        if ((authz_flags & READ_ANONYMIZED) || (authz_flags & STATUS_FOR_MONITORING)) {
+                                //XXX Intorduce a better method of keeping track if more members/types are affected
+                                olduser=events[i].any.user;
+                                events[i].any.user=(authz_flags & STATUS_FOR_MONITORING) ? NULL : anonymize_string(ctx, events[i].any.user);
+                                if (events[i].any.type==EDG_WLL_EVENT_CHANGEACL) {
+                                        oldacluser=events[i].changeACL.user_id;
+                                        events[i].changeACL.user_id=(authz_flags & STATUS_FOR_MONITORING) ? NULL : anonymize_string(ctx, events[i].changeACL.user_id);
+                                }
+                        }
+
+                        if (edg_wll_UnparseEventJSON(ctx, events + i, &event_str) != 0) goto err;
+
+                        if (olduser) {
+                                free(events[i].any.user);
+                                events[i].any.user=olduser;
+                                olduser = NULL;
+                        }
+                        if (oldacluser) {
+                                free(events[i].changeACL.user_id);
+                                events[i].changeACL.user_id=oldacluser;
+                                oldacluser = NULL;
+                        }
+                        len = strlen(event_str);
+                        newsize = size + len + HISTORY_SEPARATOR_SIZE + HISTORY_FOOTER_SIZE + 1;
+                        if (newsize > maxsize) {
+                                maxsize <<= 1;
+                                if (newsize > maxsize) maxsize = newsize;
+                                if ((tmpptr = realloc(history, maxsize)) == NULL) {
+                                        edg_wll_SetError(ctx, ENOMEM, NULL);
+                                        goto err;
+                                }
+                                history = tmpptr;
+                        }
+                        strncpy(history + size, event_str, len + 1);
+                        size += len;
+                        if (events[i+1].type) {
+                                strcpy(history + size, HISTORY_SEPARATOR);
+                                size += HISTORY_SEPARATOR_SIZE;
+                        }
+                        free(event_str);
+                        event_str = NULL;
+                }
+                strcpy(history + size, HISTORY_FOOTER);
+                size += HISTORY_FOOTER_SIZE;
+                stat->history = history;
+                history = NULL;
+        }
+
+        err:
+        free(history);
+        return edg_wll_Error(ctx, NULL, NULL);
+}
+
+//int clear_history(edg_wll_Context ctx, edg_wll_Event* events) {
+//        int i;
+//
+//        for (i = 0; events && events[i].type; i++)
+//                edg_wll_FreeEvent(&events[i]);
+//        free(events);
+//
+//       return edg_wll_Error(ctx, NULL, NULL);
+//}
+
 int edg_wll_JobStatusServer(
 	edg_wll_Context	ctx,
 	glite_jobid_const_t		job,
@@ -381,6 +481,15 @@ int edg_wll_JobStatusServer(
 			}
 		}
 #endif
+		if (flags & EDG_WLL_NOTIF_HISTORY) {
+			edg_wll_Event* events = NULL;
+			if (!(events = fetch_history(ctx, stat))) {
+				edg_wll_SetError(ctx, ctx->errCode, "Error extracting job history");
+			}
+			else {
+				collate_history(ctx, stat, events, authz_flags);
+			}
+		}
 
 		whole_cycle = 1;
 rollback:
@@ -527,6 +636,8 @@ int edg_wll_intJobStatus(
                                 case EDG_WLL_STAT_FILE_TRANSFER_COLLECTION:
                                         edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_SANDBOX_REGS);
                                         break;
+				case EDG_WLL_STAT_VIRTUAL_MACHINE:
+					edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_VM_REGS);
                                 default:
                                         glite_common_log(LOG_CATEGORY_LB_SERVER_REQUEST, LOG_PRIORITY_DEBUG, "Unknown job type, registration will not be counted in statistics.");
                                         break;
@@ -654,24 +765,6 @@ err:
 }
 
 
-/*
- * Helper for warning printouts
- */
-
-static void warn(const char* format, ...)
-{
-	va_list l;
-	va_start(l, format);
-
-	/*
-	fprintf(stderr, "Warning: ");
-	vfprintf(stderr, format, l);
-	fputc('\n', stderr);
-	*/
-
-	va_end(l);
-}
-
 static char *job_owner(edg_wll_Context ctx,char *md5_jobid)
 {
 	char	*stmt = NULL,*out = NULL;
@@ -1451,3 +1544,46 @@ cleanup:
 
 }
 
+edg_wll_ErrorCode edg_wll_getConnectedJobs(edg_wll_Context ctx, glite_jobid_const_t job, edg_wll_JobConnectionType connection, edg_wll_RelationshipRecord **connections) {
+	char *stmt;
+	glite_lbu_Statement sh = NULL;
+	int i = 0, n;
+	char *job_u;
+	char *out_stat[3];
+	glite_jobid_t	tempjobid;
+	edg_wll_RelationshipRecord *conns = NULL;
+
+	job_u = edg_wlc_JobIdGetUnique(job);
+
+	if (connection == EDG_WLL_JOBCONNECTION_UNDEFINED)
+		trio_asprintf(&stmt, "SELECT jobid_to, jobtype, connection FROM job_connections WHERE jobid_from='%s'", job_u);
+	else
+		trio_asprintf(&stmt, "SELECT jobid_to, jobtype, connection FROM job_connections WHERE jobid_from='%s' AND connection=%i", job_u, connection);
+	glite_common_log_msg(LOG_CATEGORY_LB_SERVER_DB, LOG_PRIORITY_DEBUG, stmt);
+	free(job_u);
+
+	n = edg_wll_ExecSQL(ctx,stmt,&sh);
+	free(stmt);
+
+	if (n > 0) {
+		conns = (edg_wll_RelationshipRecord*)malloc((n+1)*sizeof(edg_wll_RelationshipRecord));
+		while (edg_wll_FetchRow(ctx, sh, sizeof(out_stat)/sizeof(out_stat[0]), NULL, out_stat) == 3) {
+			glite_jobid_recreate((const char*) ctx->srvName, ctx->srvPort, out_stat[0], &tempjobid);
+			conns[i].jobid = tempjobid;
+			conns[i].jobtype = atoi(out_stat[1]);
+			conns[i].reltype = atoi(out_stat[2]);
+			free(out_stat[0]);
+			free(out_stat[1]);
+			free(out_stat[2]);
+			i++;
+		}
+		conns[i].jobid = NULL;
+		conns[i].jobtype = -1; 
+		conns[i].reltype = EDG_WLL_JOBCONNECTION_UNDEFINED;
+	}
+
+	*connections = conns;
+
+	return edg_wll_Error(ctx, NULL, NULL);
+}
+
