@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_authz.c,v 1.52 2012/06/22 12:57:28 valtri Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/lb_authz.c,v 1.58 2012/09/10 09:47:14 zsustr Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -687,9 +687,10 @@ edg_wll_InitAcl(edg_wll_Acl *acl)
 void
 edg_wll_FreeAcl(edg_wll_Acl acl)
 {
-   if ( acl->value ) GRSTgaclAclFree(acl->value);
-   if ( acl->string ) free(acl->string);
-   free(acl);
+   if (acl) {
+	   if ( acl->value ) GRSTgaclAclFree(acl->value);
+	   if ( acl->string ) free(acl->string); 
+	   free(acl); }
 }
 
 int
@@ -1227,6 +1228,25 @@ edg_wll_acl_print(edg_wll_Context ctx, edg_wll_Acl a, char **policy)
     return 0;
 }
 
+int
+equal_mapped(const char *a, const char *b, struct _edg_wll_id_mapping *mapping)
+{
+    int i;
+
+    if (mapping == NULL || mapping->num == 0)
+	return 0;
+
+    for (i = 0; i < mapping->num; i++) {
+	if (edg_wll_gss_equal_subj(a, mapping->rules[i].a) &&
+	    edg_wll_gss_equal_subj(b, mapping->rules[i].b))
+	    return 1;
+	if (edg_wll_gss_equal_subj(a, mapping->rules[i].b) &&
+	    edg_wll_gss_equal_subj(b, mapping->rules[i].a))
+	    return 1;
+    }
+    return 0;
+}
+
 int
 check_jobstat_authz(edg_wll_Context ctx,
 	    	    const edg_wll_JobStat *stat,
@@ -1246,6 +1266,9 @@ check_jobstat_authz(edg_wll_Context ctx,
 
     if (edg_wll_gss_equal_subj(peer->name, stat->owner))
 	return 1;
+    if (equal_mapped(peer->name, stat->owner, &ctx->id_mapping))
+	return 1;
+
     if (stat->payload_owner && edg_wll_gss_equal_subj(peer->name, stat->payload_owner))
 	return 1;
 
@@ -1267,3 +1290,80 @@ check_jobstat_authz(edg_wll_Context ctx,
 
     return 0;
 }
+
+int
+parse_gridmap(edg_wll_Context ctx,
+	      const char *file,
+	      struct _edg_wll_id_mapping *mapping)
+{
+    FILE *fd = NULL;
+    char line[4096];
+    char *p, *a;
+    int ret, index = 0, len, i, lineno = 0;
+
+    fd = fopen(file, "r");
+    if (fd == NULL)
+	return edg_wll_SetError(ctx, errno, "Failed to open mapping file");
+
+    /* XXX -1 */
+    while (fgets(line, sizeof(line), fd) != NULL) {
+	lineno++;
+	if ((p = strchr(line, '\n')))	// Removes trailing \n
+	    *p = '\0';
+
+	index = strspn(line, " \t");
+
+	if(strlen(line) == index) continue; // Skips empty lines
+	if(line[index]=='#') continue; // Skips commented-out lines
+
+	mapping->rules = realloc(mapping->rules, (mapping->num+1) * sizeof(_edg_wll_mapping_rule));
+	if (mapping->rules == NULL) {
+	    ret = edg_wll_SetError(ctx, ENOMEM, "Not enough memory");
+	    goto end;
+	}
+
+	for (i = 0; i < 2; i++) {
+
+		if (i) {
+			if((index + len) < strlen(line)) index = index + len + strspn(line + index + len, " \t"); // Move index to next
+			else {
+				char *errdesc;
+				asprintf(&errdesc, "Wrong format of mapping file, line %d", lineno);
+				ret = edg_wll_SetError(ctx, EINVAL, errdesc);
+				goto end;
+			}
+		}
+
+		len = (line[index] == '"' || line[index] == '\'') ?	// Get token length
+			strcspn(line + (++index), "\"'") :
+			strcspn(line + index, " ");
+
+		if(!(a = calloc(sizeof(char*), len + 1))) { // Allocate and check
+			ret = edg_wll_SetError(ctx, ENOMEM, "Not enough memory");
+			goto end;
+		}
+
+		strncpy(a, line + index, len); // Copy token contents and assign
+		edg_wll_gss_normalize_subj(a, 1);
+		if (!i) mapping->rules[mapping->num].a = a;
+		else mapping->rules[mapping->num].b = a;
+
+		if(!strlen(a)) {
+			char *errdesc;
+			asprintf(&errdesc, "Orphaned ID in mapping file, line %d", lineno);
+			ret = edg_wll_SetError(ctx, EINVAL, errdesc);
+			goto end;
+		}
+
+		if(line[index+len] == '"' || line[index+len] == '\'') len++;
+	}
+
+	mapping->num++;
+    }
+    ret = 0;
+
+end:
+    fclose(fd);
+
+    return ret;
+}
