@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_proto.c,v 1.71 2012/08/09 14:46:18 zsustr Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/lb_proto.c,v 1.81 2013/01/04 14:06:07 jfilipov Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -344,125 +344,67 @@ static void freeNotifInfo(notifInfo *ni){
 	if (ni->jobid) free(ni->jobid);
 }
 
-static int getJobsRSS(edg_wll_Context ctx, char *feedType, edg_wll_JobStat **statesOut){
+static int getJobsRSS(edg_wll_Context ctx, char *feedType, edg_wll_JobStat **statesOut, edg_wll_QueryRec **qconditions){
 	edg_wlc_JobId *jobsOut;
         //edg_wll_JobStat *statesOut;
         edg_wll_QueryRec **conds;
 	int i;
+	char *fullrssquery = NULL;
 
 	char *can_peername = edg_wll_gss_normalize_subj(ctx->peerName, 0);
 
-	if (strncmp(feedType, "finished", strlen("finished")) == 0){
-		conds = malloc(4*sizeof(*conds));
-		conds[0] = malloc(2*sizeof(**conds));
-		conds[0][0].attr = EDG_WLL_QUERY_ATTR_OWNER;
-		conds[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
-		conds[0][0].value.c = can_peername;
-		conds[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-		conds[1] = malloc(4*sizeof(**conds));
-		conds[1][0].attr = EDG_WLL_QUERY_ATTR_STATUS;
-		conds[1][0].op = EDG_WLL_QUERY_OP_EQUAL;
-		conds[1][0].value.i = EDG_WLL_JOB_DONE;
-		conds[1][1].attr = EDG_WLL_QUERY_ATTR_STATUS;
-                conds[1][1].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[1][1].value.i = EDG_WLL_JOB_ABORTED;
-		conds[1][2].attr = EDG_WLL_QUERY_ATTR_STATUS;
-                conds[1][2].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[1][2].value.i = EDG_WLL_JOB_CANCELLED;
-		conds[1][3].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-		conds[2] = malloc(2*sizeof(**conds));
-		conds[2][0].attr = EDG_WLL_QUERY_ATTR_LASTUPDATETIME;
-		conds[2][0].op = EDG_WLL_QUERY_OP_GREATER;
-		conds[2][0].value.t.tv_sec = time(NULL) - ctx->rssTime;
-		conds[2][0].value.t.tv_usec = 0;
-		conds[2][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-		conds[3] = NULL;
+	if (strcmp(feedType, "finished") == 0) { asprintf(&fullrssquery, "owner=%s&status=done|=aborted|=cancelled", can_peername); }
+	else if ((strcmp(feedType, "runningVM") == 0)) { asprintf(&fullrssquery, "owner=%s&status=running&jobtype=virtual_machine", can_peername); }
+	else if ((strcmp(feedType, "doneVM") == 0)) { asprintf(&fullrssquery, "owner=%s&status=done&donecode=ok&jobtype=virtual_machine", can_peername); }
+	else if (strcmp(feedType, "running") == 0) { asprintf(&fullrssquery, "owner=%s&status=running", can_peername); }
+	else if (strcmp(feedType, "aborted") == 0) { asprintf(&fullrssquery, "owner=%s&status=aborted", can_peername); }
+
+	if (fullrssquery) {
+		i = edg_wll_ParseQueryConditions(ctx, fullrssquery, &conds);
+		free(fullrssquery);
+		if (i) return i;
 	}
-	else if (strncmp(feedType, "running", strlen("running")) == 0){
-                conds = malloc(4*sizeof(*conds));
-                conds[0] = malloc(2*sizeof(**conds));
-                conds[0][0].attr = EDG_WLL_QUERY_ATTR_OWNER;
-                conds[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[0][0].value.c = can_peername;
-                conds[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[1] = malloc(2*sizeof(**conds));
-                conds[1][0].attr = EDG_WLL_QUERY_ATTR_STATUS;
-                conds[1][0].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[1][0].value.i = EDG_WLL_JOB_RUNNING;
-                conds[1][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[2] = malloc(2*sizeof(**conds));
-                conds[2][0].attr = EDG_WLL_QUERY_ATTR_LASTUPDATETIME;
-                conds[2][0].op = EDG_WLL_QUERY_OP_GREATER;
-                conds[2][0].value.t.tv_sec = time(NULL) - ctx->rssTime;
-                conds[2][0].value.t.tv_usec = 0;
-                conds[2][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[3] = NULL;
-        }
-	else if (strncmp(feedType, "aborted", strlen("aborted")) == 0){
-                conds = malloc(4*sizeof(*conds));
-                conds[0] = malloc(2*sizeof(**conds));
-                conds[0][0].attr = EDG_WLL_QUERY_ATTR_OWNER;
-                conds[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[0][0].value.c = can_peername;
-                conds[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[1] = malloc(2*sizeof(**conds));
-                conds[1][0].attr = EDG_WLL_QUERY_ATTR_STATUS;
-                conds[1][0].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[1][0].value.i = EDG_WLL_JOB_ABORTED;
-                conds[1][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[2] = malloc(2*sizeof(**conds));
-                conds[2][0].attr = EDG_WLL_QUERY_ATTR_LASTUPDATETIME;
-                conds[2][0].op = EDG_WLL_QUERY_OP_GREATER;
-                conds[2][0].value.t.tv_sec = time(NULL) - ctx->rssTime;
-                conds[2][0].value.t.tv_usec = 0;
-                conds[2][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[3] = NULL;
-        }
-	else{
-		*statesOut = NULL;
-		free(can_peername);
-		return -1;
+	else {  
+		if (!qconditions) {
+			*statesOut = NULL;
+			free(can_peername);
+			edg_wll_SetError(ctx, EINVAL, "Invalid RSS feed specified");
+			return EINVAL;
+		}
+		else {
+			conds = qconditions;
+		}
 	}
 
+	for ( i=0; conds[i]; i++ ); // Look up the trailing position in the array: the time condition will be added to it
+
+	glite_common_log(LOG_CATEGORY_LB_SERVER_REQUEST, LOG_PRIORITY_DEBUG, "RSS: adding time condition (No. %d) to query. rssTime = %ld", i, (long)ctx->rssTime);
+
+	conds = realloc(conds,(i+2)*sizeof(*conds)); // Extend array by 1
+	
+	conds[i] = malloc(2*sizeof(**conds));
+	conds[i][0].attr = EDG_WLL_QUERY_ATTR_LASTUPDATETIME;
+	conds[i][0].op = EDG_WLL_QUERY_OP_GREATER;
+	conds[i][0].value.t.tv_sec = time(NULL) - ctx->rssTime;
+	conds[i][0].value.t.tv_usec = 0;
+	conds[i][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
+	conds[i+1] = NULL;
+
+
 	if (edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)conds, 0, &jobsOut, statesOut)){
 		*statesOut = NULL;
 	}
 
-	for (i = 0; conds[i]; i++)
-		free(conds[i]);
-	free(conds);
+	if (!qconditions) {
+		for (i = 0; conds[i]; i++)
+			free(conds[i]);
+		free(conds);
+	}
 	free(can_peername);
 
 	return 0;
 }
 
-static int getVMsOnHost(edg_wll_Context ctx, char *hostname, edg_wll_JobStat **statesOut){
-        edg_wlc_JobId *jobsOut;
-        edg_wll_QueryRec **conds;
-        int i;
-
-                conds = malloc(2*sizeof(*conds));
-                conds[0] = malloc(2*sizeof(**conds));
-                conds[0][0].attr = EDG_WLL_QUERY_ATTR_DESTINATION;
-                conds[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
-                conds[0][0].value.c = hostname;
-                conds[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
-                conds[1] = NULL;
-	
-
-        if (edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)conds, 0, &jobsOut, statesOut)){
-                *statesOut = NULL;
-        }
-
-        for (i = 0; conds[i]; i++)
-                free(conds[i]);
-        free(conds);
-
-        return 0;
-}
-
-
-
 static void hup_handler(int sig) {
 	purge_quit = 1;
 }
@@ -618,6 +560,13 @@ int edg_wll_ParseQueryConditions(edg_wll_Context ctx, const char *query, edg_wll
 						goto err;
 					}
 					break;
+				case EDG_WLL_QUERY_ATTR_VM_STATUS:
+					if ( 0 > (conds[i][j].value.i = edg_wll_StringToVMStat(value))) {
+                                                asprintf(&errmsg, "Unknown VM job state \"%s\" in query", value);
+                                                err = edg_wll_SetError(ctx, EINVAL, errmsg);
+                                                goto err;
+                                        }
+					break;
 				default:
 					asprintf(&errmsg, "Value conversion for attribute \"%s\" not supported in current implementation", attribute);
 					err = edg_wll_SetError(ctx, ENOSYS, errmsg);
@@ -844,11 +793,11 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 	char **response,char ***headersOut,char **bodyOut,
 	int *httpErr)
 {
-	char *requestPTR = NULL, *message = NULL, *requestMeat = NULL, *querystr, *queryconds = NULL;
+	char *requestPTR = NULL, *message = NULL, *requestMeat = NULL, *querystr, *queryconds = NULL, *flagstr, *queryflags = NULL, *feedType = NULL;
 	int	ret = HTTP_OK;
 	int 	html = outputHTML(headers);
 	int 	text = 0; //XXX: Plain text communication is special case of html here, hence when text=1, html=1 too
-	int	i, j;
+	int	i, j, rflags = 0;
 	int	isadm;
 	http_admin_option adm_opt = HTTP_ADMIN_OPTION_MY;
 	http_extra_option extra_opt = HTTP_EXTRA_OPTION_NONE;
@@ -924,6 +873,18 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 					}
 				}
 			}
+
+                        if ((flagstr = strstr(requestPTR, "?flags="))) {
+                                int len = strcspn(flagstr+strlen("?flags="),"? \f\n\r\t\v");
+                                if (len) {
+                                        queryflags = (char*)calloc((len+1),sizeof(char));
+                                        queryflags = strncpy(queryflags, flagstr+strlen("?flags="), len);
+					glite_common_log(LOG_CATEGORY_LB_SERVER_REQUEST, LOG_PRIORITY_DEBUG, "HTML query flags: \"%s\"", queryflags);
+					rflags = edg_wll_string_to_stat_flags(queryflags);
+					free(queryflags);
+                                }
+                        }
+
 			strip_request_of_queries(requestPTR);
 		}
 
@@ -940,13 +901,10 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 		else if ((!strcmp(requestMeat, "/")) && ((extra_opt == HTTP_EXTRA_OPTION_NONE) || ( extra_opt == HTTP_EXTRA_OPTION_QUERY))) {
                 	edg_wlc_JobId *jobsOut = NULL;
 			edg_wll_JobStat *statesOut = NULL;
-			int	i, flags;
-
-			// XXX: query strings are now not recognized as flags. Needs modifying.
-			flags = (requestPTR[1]=='?') ? edg_wll_string_to_stat_flags(requestPTR + 2) : 0;
+			int	i;
 
 			switch ( extra_opt == HTTP_EXTRA_OPTION_QUERY ?
-                                edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)job_conditions, flags, &jobsOut, &statesOut) :
+                                edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)job_conditions, rflags, &jobsOut, &statesOut) :
                                 edg_wll_UserJobsServer(ctx, EDG_WLL_STAT_CHILDREN, &jobsOut, &statesOut)) {
 
 				case 0: edg_wll_UserInfoToHTML(ctx, jobsOut, statesOut, &message, text);
@@ -969,7 +927,7 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 	        } 
 
 	/* GET /[jobId]: Job Status */
-		else if (strcmp(requestMeat, "/RSS:")
+		else if (strncmp(requestMeat, "/RSS:", 5)
 			&& strcmp(requestMeat, "/NOTIF")
 			&& strncmp(requestMeat, "/NOTIF:", 7)
 			&& strncmp(requestMeat, "/VMHOST:", 8)
@@ -999,30 +957,11 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 				edg_wll_SetError(ctx,EDG_WLL_ERROR_JOBID_FORMAT,fullid);
 				ret = HTTP_BADREQ;
 			}
-			else switch (edg_wll_JobStatusServer(ctx,jobId,EDG_WLL_STAT_CLASSADS | EDG_WLL_STAT_CHILDREN,&stat)) {
-				case 0: if (text) { 
-						edg_wll_JobStatusToText(ctx,stat,&message); 
-						edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_TEXT_VIEWS);
-					}
-					else if (html) {
-						edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_HTML_VIEWS);
-						switch(stat.jobtype){
-						case EDG_WLL_STAT_CREAM:
-							edg_wll_CreamJobStatusToHTML(ctx,stat,&message);
-							break;
-						case EDG_WLL_STAT_FILE_TRANSFER:
-						case EDG_WLL_STAT_FILE_TRANSFER_COLLECTION:
-							edg_wll_FileTransferStatusToHTML(ctx,stat,&message);
-							break;
-						default:
-							//XXX need some more implementations
-							edg_wll_GeneralJobStatusToHTML(ctx,stat,&message);
-							break;
-						}
-					}
-					
-					else ret = HTTP_OK;
-					break;
+			else switch (edg_wll_JobStatusServer(ctx,jobId,EDG_WLL_STAT_CLASSADS | EDG_WLL_STAT_CHILDREN | rflags, &stat)) {
+				case 0:
+					edg_wll_GeneralJobStatusToHTML(ctx,stat,&message,text);
+					edg_wll_ServerStatisticsIncrement(ctx, text ? SERVER_STATS_TEXT_VIEWS : SERVER_STATS_HTML_VIEWS);
+					ret = HTTP_OK; break;
 				case ENOENT: ret = HTTP_NOTFOUND; break;
 				case EINVAL: ret = HTTP_INVALID; break;
 				case EPERM : ret = HTTP_UNAUTH; break;
@@ -1069,28 +1008,32 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 			
 			free(pomCopy);
 
-			if (text) {
-				edg_wll_NotificationToText(ctx, &ni, &message);
-				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_TEXT_VIEWS);
-			}
-			else {
-				edg_wll_NotificationToHTML(ctx, &ni, &message);
-				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_HTML_VIEWS);
-			}
+			edg_wll_NotificationToHTML(ctx, &ni, &message, text);
+
+			edg_wll_ServerStatisticsIncrement(ctx, text ? SERVER_STATS_TEXT_VIEWS : SERVER_STATS_HTML_VIEWS);
 
 			freeNotifInfo(&ni);
 
 	/*GET /RSS:[feed type] RSS feed */
 		} else if (strncmp(requestPTR, "/RSS:", strlen("/RSS:")) == 0){
 			edg_wll_JobStat *states;
-			char *feedType;
 			int i;
 			int idx;
 
-			feedType = requestPTR + strlen("/RSS:");
-			if (getJobsRSS(ctx, feedType, &states) < 0){
-				ret = HTTP_INTERNAL;
-                                goto err;
+			feedType = strdup(requestPTR + strlen("/RSS:"));
+			feedType[strrchr(feedType, ' ')-feedType] = 0;
+			switch(getJobsRSS(ctx, feedType, &states, (edg_wll_QueryRec **)job_conditions)) {
+				case 0:
+					break;
+				case EINVAL:
+					ret = HTTP_INVALID;
+                                	goto err;
+				case ENOSYS:
+					ret = HTTP_NOTIMPL;
+                                	goto err;
+				default:
+					return HTTP_BADREQ;
+                                	goto err;
 			}
 
 			// check if owner and lastupdatetime is indexed
@@ -1108,22 +1051,6 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 				edg_wll_RSSFeed(ctx, states, requestPTR, &message);
 				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_RSS_VIEWS);
 			}
-	/*GET /VMHOST:[hostname] all VMs associated with hostname */
-		} else if (strncmp(requestMeat, "/VMHOST:", strlen("/VMHOST:")) == 0) {
-			char *host = requestMeat + strlen("/VMHOST:");
-			edg_wll_JobStat *states;
-			if (getVMsOnHost(ctx, host, &states)){
-				ret = HTTP_INTERNAL;
-                                goto err;
-			}
-			if (text){
-//				edg_wll_VMHostToText(ctx, requestPTR + strlen("/VMHOST:"), states, &message);
-//				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_TEXT_VIEWS);
-			}
-			else{
-				edg_wll_VMHostToHTML(ctx, host, states, &message);
-				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_HTML_VIEWS);
-			}
 		} else if (!strcmp(requestMeat, "/favicon.ico")) {
 			message=NULL;
 			ret = HTTP_NOTFOUND;
@@ -1162,14 +1089,14 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 		} else if (extra_opt == HTTP_EXTRA_OPTION_CONFIGURATION) {
 			// also browser-readable HTML version here?
 			isadm = ctx->noAuth || edg_wll_amIroot(ctx->peerName, ctx->fqans,&ctx->authz_policy);
-			edg_wll_ConfigurationToText(ctx, isadm, &message);
+			edg_wll_ConfigurationToHTML(ctx, isadm, &message, text);
 			edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_TEXT_VIEWS);
 	/* GET /?stats*/
 		} else if (extra_opt == HTTP_EXTRA_OPTION_STATS) {
 			isadm = ctx->noAuth || edg_wll_amIroot(ctx->peerName, ctx->fqans,&ctx->authz_policy);
 			if ( !isadm && ctx->count_server_stats == 2 ) {
 				ret = HTTP_UNAUTH;
-				edg_wll_SetError(ctx, EPERM, "Only superusers can view server usage statistics on this particular server.");
+				edg_wll_SetError(ctx, EPERM, "Only superusers can view server usage statistics on this server.");
 			}
 			else {
 				edg_wll_StatisticsToHTML(ctx, &message, text);
@@ -1684,6 +1611,7 @@ err:	asprintf(response,"HTTP/1.1 %d %s",ret,edg_wll_HTTPErrorMessage(ret));
 	}
 
 	free(queryconds);
+	free(feedType);
 
 	if (requestPTR) free(requestPTR);
 	if (requestMeat) free(requestMeat);
