@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/query.c,v 1.63 2012/08/08 15:48:40 zsustr Exp $"
+#ident "$Header: /cvs/glite/org.glite.lb.server/src/query.c,v 1.67 2013/01/04 14:06:07 jfilipov Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -82,7 +82,6 @@ int edg_wll_QueryEventsServer(
 					ret = 0,
 					offset = 0, limit = 0,
 					limit_loop = 1,
-					eperm = 0,
 					where_flags = 0;
 	char *peerid = NULL;
 	char *can_peername = NULL, *can_peerid = NULL;
@@ -221,7 +220,6 @@ int edg_wll_QueryEventsServer(
 					if ( edg_wll_JobStatusServer(ctx, out[i].any.jobId, 0, &state_out) )
 					{
 						edg_wll_FreeEvent(out+i);
-						if (edg_wll_Error(ctx,NULL,NULL) == EPERM) eperm = 1;
 						goto fetch_cycle_cleanup;
 					}
 				
@@ -253,7 +251,6 @@ int edg_wll_QueryEventsServer(
 					ret = edg_wll_GetACL(ctx, out[i].any.jobId, &acl);
 					free(jobid);
 					if (ret || acl == NULL) {
-						eperm = 1;
 						edg_wll_FreeEvent(out+i);
 						edg_wll_ResetError(ctx); /* XXX: should be reported somewhere at least in debug mode */
 						goto fetch_cycle_cleanup;
@@ -262,7 +259,6 @@ int edg_wll_QueryEventsServer(
 					ret = edg_wll_CheckACL(ctx, acl, EDG_WLL_CHANGEACL_READ);
 					edg_wll_FreeAcl(acl);
 					if (ret) {
-						eperm = 1;
 						edg_wll_FreeEvent(out+i);
 						edg_wll_ResetError(ctx); /* XXX: should be reported somewhere at least in debug mode */
 						goto fetch_cycle_cleanup;
@@ -296,9 +292,7 @@ limit_cycle_cleanup:
 		glite_lbu_FreeStmt(&sh);
 	} while ( limit_loop );
 
-	if ( i == 0 && eperm )
-		edg_wll_SetError(ctx, EPERM, "matching events found but authorization failed");
-	else if ( i == 0 )
+	if ( i == 0 )
 		edg_wll_SetError(ctx, ENOENT, "no matching events found");
 	else
 	{
@@ -454,7 +448,6 @@ int edg_wll_QueryJobsServerStream(
 	int					i = 0,
 						j = 0,
 						ret = 0,
-						eperm = 0,
 						eidrm = 0,
 						limit = 0, offset = 0,
 						limit_loop = 1,
@@ -569,7 +562,6 @@ int edg_wll_QueryJobsServerStream(
 				if ( edg_wll_JobStatusServer(ctx, jobid, (where_flags & FL_SEL_JDL)?(flags|EDG_WLL_STAT_CLASSADS):flags, &status) )
 				{
 					glite_jobid_free(jobid);
-					if (edg_wll_Error(ctx,NULL,NULL) == EPERM) eperm = 1;
 					goto fetch_cycle_cleanup;
 				}
 
@@ -587,7 +579,6 @@ int edg_wll_QueryJobsServerStream(
 #if 0
 			if ( !ctx->noAuth && (!ctx->peerName || strcmp(res[1], strmd5(ctx->peerName, NULL))) )
 			{
-				eperm = 1;
 				glite_jobid_free(jobid);
 				edg_wll_FreeStatus(&status);
 				goto fetch_cycle_cleanup;
@@ -674,12 +665,15 @@ limit_cycle_cleanup:
 	}
 
 	if ( !n ) {
-		if (eperm) edg_wll_SetError(ctx, EPERM, "matching jobs found but authorization failed");
-		else {
-			if (eidrm) edg_wll_SetError(ctx, EIDRM, "matching job already purged");
-			else edg_wll_SetError(ctx, ENOENT, "no matching jobs found");
-		}
+		if (eidrm) edg_wll_SetError(ctx, EIDRM, "matching job already purged");
+		else edg_wll_SetError(ctx, ENOENT, "no matching jobs found");
 	}
+	else {
+		// There was permission problem but there were still some results
+		glite_common_log(LOG_CATEGORY_LB_SERVER, LOG_PRIORITY_DEBUG, "Results were returned for query, but also error code %d. Resetting.", edg_wll_Error(ctx,NULL,NULL));
+		if (edg_wll_Error(ctx,NULL,NULL) == EPERM) edg_wll_ResetError(ctx);
+	}
+
 
 cleanup:
 	// finish
@@ -848,6 +842,15 @@ static char *ec_to_head_where(edg_wll_Context ctx,const edg_wll_QueryRec **ec)
 			ct++;
 			break;
 
+		case EDG_WLL_QUERY_ATTR_VM_STATUS:
+			if ( ec[m][n].op != EDG_WLL_QUERY_OP_EQUAL && ec[m][n].op != EDG_WLL_QUERY_OP_UNEQUAL )
+			{
+				edg_wll_SetError(ctx, EINVAL, "only `=' and '!=' supported with VM status");
+				return NULL;
+			}
+			ct++;
+			break;
+
 		default:
 			sprintf(msg, "ec_to_head_where(): attr=%d unsupported", ec[m][n].attr);
 			edg_wll_SetError(ctx, EINVAL, msg);
@@ -1153,6 +1156,15 @@ static char *jc_to_head_where(
 			ct++;
 			break;
 
+		case EDG_WLL_QUERY_ATTR_VM_STATUS:
+                        if ( jc[m][n].op != EDG_WLL_QUERY_OP_EQUAL && jc[m][n].op != EDG_WLL_QUERY_OP_UNEQUAL )
+                        {
+                                edg_wll_SetError(ctx, EINVAL, "only `=' and '!=' supported with VM status");
+                                return NULL;
+                        }
+                        ct++;
+                        break;
+
 
 		default:
 			sprintf(msg, "jc_to_head_where(): attr=%d unsupported", jc[m][n].attr);
@@ -1288,6 +1300,7 @@ static char *jc_to_head_where(
 		case EDG_WLL_QUERY_ATTR_EXITCODE:
 		case EDG_WLL_QUERY_ATTR_STATUS:
 		case EDG_WLL_QUERY_ATTR_JOB_TYPE:
+		case EDG_WLL_QUERY_ATTR_VM_STATUS:
 			if (   !is_indexed(&(jc[m][n]), ctx)
 				|| !(cname = edg_wll_QueryRecToColumn(&(jc[m][n]))) )
 			{
@@ -1663,6 +1676,7 @@ int match_status(edg_wll_Context ctx, const edg_wll_JobStat *oldstat, const edg_
 					} else if ( conds[i][j].op == EDG_WLL_QUERY_OP_UNEQUAL ) goto or_satisfied;
 					// TODO: EDG_WLL_QUERY_OP_LESS, EDG_WLL_QUERY_OP_GREATER, EDG_WLL_QUERY_OP_WITHIN, EDG_WLL_QUERY_OP_CHANGED
 				}
+				break;
 			case EDG_WLL_QUERY_ATTR_NETWORK_SERVER:
 				if ( stat->network_server )
 				{
@@ -1802,6 +1816,7 @@ int match_status(edg_wll_Context ctx, const edg_wll_JobStat *oldstat, const edg_
 					if (oldstat->stateEnterTime.tv_sec != stat->stateEnterTime.tv_sec) goto or_satisfied;
 					break;
 				}
+				break;
 			case EDG_WLL_QUERY_ATTR_LASTUPDATETIME:
 				if ( !stat->lastUpdateTime.tv_sec )
 					break;
@@ -1827,6 +1842,7 @@ int match_status(edg_wll_Context ctx, const edg_wll_JobStat *oldstat, const edg_
 					if (oldstat->lastUpdateTime.tv_sec != stat->lastUpdateTime.tv_sec) goto or_satisfied;
 					break;
 				}
+				break;
 			case EDG_WLL_QUERY_ATTR_JOB_TYPE: 
 				switch ( conds[i][j].op )
 				{
@@ -1837,6 +1853,18 @@ int match_status(edg_wll_Context ctx, const edg_wll_JobStat *oldstat, const edg_
 					if ( conds[i][j].value.i != stat->jobtype ) goto or_satisfied;
 					break;
 				}
+				break;
+			case EDG_WLL_QUERY_ATTR_VM_STATUS:
+				switch ( conds[i][j].op )
+                                {
+                                case EDG_WLL_QUERY_OP_EQUAL:
+                                        if ( conds[i][j].value.i == stat->vm_state ) goto or_satisfied;
+                                        break;
+                                case EDG_WLL_QUERY_OP_UNEQUAL:
+                                        if ( conds[i][j].value.i != stat->vm_state ) goto or_satisfied;
+                                        break;
+                                }
+				break;
 			default:
 				break;
 			}
